(*** OPENS ********************************************************************)
(* open Util;; *)

(*** GLOBALS ******************************************************************)
let fresh_counter = ref ~-1;;

(*** TYPES ********************************************************************)
type p = 
 | Bot
 | Top
 | P of int
 | Fresh of int 
 | Not of p
 | And of p * p
 | Or of p * p
 | Iff of p * p
;;

(*** FUNCTIONS ****************************************************************)

(*** printers ***)
let rec fprintf ppf = function
 | Top -> Format.fprintf ppf "true"
 | Bot -> Format.fprintf ppf "false"
 | P x -> Format.fprintf ppf "p%d" x
 | Fresh x -> Format.fprintf ppf "f%d" x
 | Not x -> Format.fprintf ppf "(not %a)" fprintf x
 | And (x,y) -> Format.fprintf ppf "(and %a %a)" fprintf x fprintf y
 | Or (x,y) -> Format.fprintf ppf "(or %a %a)" fprintf x fprintf y
 | Iff (x,y) -> Format.fprintf ppf "(iff %a %a)" fprintf x fprintf y
;;

let to_string f =
 Format.fprintf Format.str_formatter "%a" fprintf f;
 Format.flush_str_formatter ()
;;

(*** predicates ***)
let is_atom = function
 | P _ | Fresh _ -> true
 | _ -> false
;;

let is_literal = function 
 | Not a | a -> is_atom a
;;

(*** constructors ***)
let var x = P x;;
let fresh () = 
 incr fresh_counter;
 Fresh (!fresh_counter);
;;

(*** constants ***)
let top = Top;;
let bot = Bot;;

(*** primitive operators ***)
let (<&>) x y = 
 if x = bot || y = bot then bot 
 else if x = top then y
 else if y = top then x
 else And (x,y);;

let (<|>) x y = 
 if x = top || y = top then top 
 else if x = bot then y
 else if y = bot then x
 else Or (x,y)
;;

let (!!) x = if x = bot then top else if x = top then bot else Not x;;


let (<<->>) x y = if x = top then y else if x = bot then !!y 
             else if y = top then x else if y = bot then !!x
             else Iff (x,y);;

(*** derived operators ***)
let (<->>) x y = (!!x) <|> y;;
let (<<->) x y = y <->> x;;
let (<+>) x y =  x <<->> !!y;;

(*** useful operators ***)
let big_and = List.fold_left (<&>) top;;
let big_or = List.fold_left (<|>) bot;;


(*** debug output ***)
let print_formula f = 
 if !Debug.formula then (
  Debug.print_time ();
  Format.printf "formula: @[<1>%a@]@\n" fprintf f;
 );;

