(*** OPENS ********************************************************************)
open Formula;;

(*** MODULES ******************************************************************)
module I = Interface;;
module Idx = Index;;

(*** FUNCTIONS ****************************************************************)

(********************** calling Minisat Interface begin ***********************)
(*map literal to integer (DIMACS format)*)
let rec lit2int idx = function
 | P x | Fresh x as v ->  Idx.index idx v
 | Not l when Formula.is_atom l -> ~-(lit2int idx l)
 | _             -> (* non-literal formula *) 
  raise (Match_failure ("miniSat.lit2int", 0, 0))
;;

let to_dimacs clause vars = Array.of_list (List.map (lit2int vars) clause);;

let send_clause idx s conjunct = 
 let clause = to_dimacs conjunct idx in
 I.add s clause (Array.length clause) 
;;

let solve cnf =
 let idx = Idx.make 1 in
 let s = I.get_solver () in
 List.iter (send_clause idx s) cnf;
 let answer = 
  if I.is_sat s = 1 then begin
  let assign = List.fold_left
   (fun ass x -> (x,I.val_of s (Idx.index idx x) = 1)::ass)
   []
   (Idx.values idx)
  in
   Some assign
  end else
   None
 in
 ignore (I.delete_solver s);
 answer
;;
